@@ -8,8 +8,10 @@ from packaging import version
 from typing import List
 
 from .api_key import ApiKey
+from .batch_dataset import BatchDataset
 from .batch_prediction import BatchPrediction
 from .schema import Schema
+from .data_filter import DataFilter
 from .database_connector import DatabaseConnector
 from .dataset import Dataset
 from .dataset_version import DatasetVersion
@@ -46,7 +48,7 @@ class ApiException(Exception):
 
 
 class ApiClient():
-    client_version = '0.13.0'
+    client_version = '0.13.5'
 
     def __init__(self, api_key=None, server='https://abacus.ai'):
         self.api_key = api_key
@@ -114,7 +116,7 @@ class ApiClient():
         elif method == 'PUT':
             return requests.put(url, params=query_params, data=body, headers=headers, files=files, timeout=90)
         elif method == 'PATCH':
-            return requests.patch(url, params=query_params, data=body, headers=headers, timeout=90)
+            return requests.patch(url, params=query_params, json=body, headers=headers, files=files, timeout=90)
         elif method == 'DELETE':
             return requests.delete(url, params=query_params, data=body, headers=headers)
         else:
@@ -192,7 +194,7 @@ class ApiClient():
 
     def rename_project(self, project_id: str, name: str):
         '''This method renames a project after it is created. To rename the project, specify its ID and new name.    '''
-        return self._call_api('renameProject', 'PATCH', query_params={'projectId': project_id, 'name': name})
+        return self._call_api('renameProject', 'PATCH', query_params={}, body={'projectId': project_id, 'name': name})
 
     def set_column_data_type(self, project_id: str, dataset_id: str, column: str, data_type: str):
         '''Set a column's type in a specified dataset. Specify the project ID, dataset ID, column name and data type, and the method will return the entire dataset with the resulting changes reflected.    '''
@@ -202,6 +204,22 @@ class ApiClient():
         '''Set a column's column mapping. If the column mapping is single-use and already set in another column in this dataset, this call will first remove the other column's mapping and move it to this column. The model returns a list of all schemas for each column with the reflected changes.    '''
         return self._call_api('setColumnMapping', 'POST', query_params={'datasetId': dataset_id}, body={'projectId': project_id, 'column': column, 'columnMapping': column_mapping}, parse_type=Schema)
 
+    def add_custom_column(self, project_id: str, dataset_id: str, column: str, sql: str):
+        '''Adds a custom column to the dataset    '''
+        return self._call_api('addCustomColumn', 'POST', query_params={'datasetId': dataset_id}, body={'projectId': project_id, 'column': column, 'sql': sql}, parse_type=Schema)
+
+    def edit_custom_column(self, project_id: str, dataset_id: str, column: str, new_column_name: str = None, sql: str = None):
+        '''Edits a custom column    '''
+        return self._call_api('editCustomColumn', 'PATCH', query_params={'datasetId': dataset_id}, body={'projectId': project_id, 'column': column, 'newColumnName': new_column_name, 'sql': sql}, parse_type=Schema)
+
+    def delete_custom_column(self, project_id: str, dataset_id: str, column: str):
+        '''Deletes a custom column    '''
+        return self._call_api('deleteCustomColumn', 'DELETE', query_params={'projectId': project_id, 'datasetId': dataset_id, 'column': column}, parse_type=Schema)
+
+    def set_project_dataset_filters(self, project_id: str, dataset_id: str, filters: list):
+        '''Sets the data filters for a dataset uploaded under a project.    '''
+        return self._call_api('setProjectDatasetFilters', 'POST', query_params={'datasetId': dataset_id}, body={'projectId': project_id, 'filters': filters})
+
     def validate_project(self, project_id: str):
         '''Validates that the specified project has all required datasets for its use case and that all datasets attached to the project have all required columns.    '''
         return self._call_api('validateProject', 'GET', query_params={'projectId': project_id}, parse_type=ProjectValidation)
@@ -239,13 +257,21 @@ class ApiClient():
         return self._call_api('createDatasetFromLocalFile', 'POST', query_params={}, body={'name': name, 'fileFormat': file_format, 'projectId': project_id, 'datasetType': dataset_type}, parse_type=Upload)
 
     def create_dataset_version_from_local_file(self, dataset_id: str, file_format: str = None):
-        '''Creates a new version of the specified dataset using a local file upload. The method will return the attributes of the dataset.    '''
+        '''Creates a new version of the specified dataset using a local file upload.    '''
         return self._call_api('createDatasetVersionFromLocalFile', 'POST', query_params={'datasetId': dataset_id}, body={'fileFormat': file_format}, parse_type=Upload)
 
+    def create_dataset_version_from_database_connector(self, dataset_id: str, object_name: str = None, columns: str = None, query_arguments: str = None):
+        '''Creates a new version of the specified dataset    '''
+        return self._call_api('createDatasetVersionFromDatabaseConnector', 'POST', query_params={'datasetId': dataset_id}, body={'objectName': object_name, 'columns': columns, 'queryArguments': query_arguments}, parse_type=DatasetVersion)
+
     def create_streaming_dataset(self, name: str, project_id: str, dataset_type: str):
         '''Creates a streaming dataset. Use a streaming dataset if your dataset is receiving information from multiple sources over an extended period of time.    '''
         return self._call_api('createStreamingDataset', 'POST', query_params={}, body={'name': name, 'projectId': project_id, 'datasetType': dataset_type}, parse_type=Dataset)
 
+    def snapshot_streaming_data(self, dataset_id: str):
+        '''Snapshots the current data in the streaming dataset for training.    '''
+        return self._call_api('snapshotStreamingData', 'POST', query_params={'datasetId': dataset_id}, body={}, parse_type=DatasetVersion)
+
     def get_file_connector_instructions(self, bucket: str, write_permission: bool = False):
         '''Retrieves verification information to create a data connector.    '''
         return self._call_api('getFileConnectorInstructions', 'GET', query_params={'bucket': bucket, 'writePermission': write_permission}, parse_type=FileConnectorInstructions)
@@ -266,6 +292,10 @@ class ApiClient():
         '''Get the schema of an object in an database connector.    '''
         return self._call_api('getDatabaseConnectorObjectSchema', 'GET', query_params={'databaseConnectorId': database_connector_id, 'objectName': object_name})
 
+    def rename_database_connector(self, database_connector_id: str, name: str):
+        '''Renames a Database Connector    '''
+        return self._call_api('renameDatabaseConnector', 'PATCH', query_params={}, body={'databaseConnectorId': database_connector_id, 'name': name})
+
     def verify_database_connector(self, database_connector_id: str):
         '''Checks to see if Abacus.AI can access the database.    '''
         return self._call_api('verifyDatabaseConnector', 'GET', query_params={'databaseConnectorId': database_connector_id})
@@ -286,6 +316,18 @@ class ApiClient():
         '''Creates a streaming token for the specified project. Streaming tokens are used to authenticate requests to append data to streaming datasets.    '''
         return self._call_api('createStreamingToken', 'POST', query_params={}, body={}, parse_type=StreamingAuthToken)
 
+    def set_ephemeral(self, dataset_id: str, ephemeral: bool = None):
+        '''If set, the streaming dataset will be ignored during training.    '''
+        return self._call_api('setEphemeral', 'POST', query_params={'datasetId': dataset_id}, body={'ephemeral': ephemeral})
+
+    def set_ignore_before(self, dataset_id: str, timestamp: int = None):
+        '''If set, the streaming dataset will ignore all entries sent before the timestamp.    '''
+        return self._call_api('setIgnoreBefore', 'POST', query_params={'datasetId': dataset_id}, body={'timestamp': timestamp})
+
+    def set_lookback_days(self, dataset_id: str, lookback_days: int = None):
+        '''If set, the streaming dataset will ignore all entries sent before the specified number of days.    '''
+        return self._call_api('setLookbackDays', 'POST', query_params={'datasetId': dataset_id}, body={'lookbackDays': lookback_days})
+
     def list_streaming_tokens(self):
         '''Retrieves a list of all streaming tokens along with their attributes.    '''
         return self._call_api('listStreamingTokens', 'GET', query_params={}, parse_type=StreamingAuthToken)
@@ -324,7 +366,7 @@ class ApiClient():
 
     def rename_dataset(self, dataset_id: str, name: str):
         '''Rename a dataset that has already been defined. Specify the new name and dataset ID, and the model will return the attributes of the renamed dataset.    '''
-        return self._call_api('renameDataset', 'POST', query_params={'datasetId': dataset_id}, body={'name': name})
+        return self._call_api('renameDataset', 'PATCH', query_params={'datasetId': dataset_id}, body={'name': name})
 
     def delete_dataset(self, dataset_id: str):
         '''Deletes the specified dataset from the organization.    '''
@@ -346,6 +388,10 @@ class ApiClient():
         '''Retrieves a full description of the specified model.    '''
         return self._call_api('describeModel', 'GET', query_params={'modelId': model_id}, parse_type=Model)
 
+    def update_model_training_config(self, model_id: str, training_config: dict):
+        '''Edits the model's traning config    '''
+        return self._call_api('updateModelTrainingConfig', 'PATCH', query_params={}, body={'modelId': model_id, 'trainingConfig': training_config}, parse_type=Model)
+
     def get_model_metrics(self, model_id: str, model_version: str = None, baseline_metrics: bool = False):
         '''Retrieves a full list of the metrics for the specified model.    '''
         return self._call_api('getModelMetrics', 'GET', query_params={'modelId': model_id, 'modelVersion': model_version, 'baselineMetrics': baseline_metrics}, parse_type=ModelMetrics)
@@ -392,7 +438,7 @@ class ApiClient():
 
     def update_deployment(self, deployment_id: str, name: str = None, description: str = None):
         '''Updates a deployment's name and/or description.    '''
-        return self._call_api('updateDeployment', 'PATCH', query_params={'deploymentId': deployment_id, 'name': name, 'description': description})
+        return self._call_api('updateDeployment', 'PATCH', query_params={'deploymentId': deployment_id}, body={'name': name, 'description': description})
 
     def start_deployment(self, deployment_id: str):
         '''Restarts the specified deployment that was previously suspended.    '''
@@ -400,7 +446,7 @@ class ApiClient():
 
     def set_deployment_model_version(self, deployment_id: str, model_version: str):
         '''Promotes a Model Version to be served in the Deployment    '''
-        return self._call_api('setDeploymentModelVersion', 'PATCH', query_params={'deploymentId': deployment_id, 'modelVersion': model_version})
+        return self._call_api('setDeploymentModelVersion', 'PATCH', query_params={'deploymentId': deployment_id}, body={'modelVersion': model_version})
 
     def stop_deployment(self, deployment_id: str):
         '''Stops the specified deployment.    '''
@@ -442,9 +488,9 @@ class ApiClient():
         '''Returns a list of forecasts for a given entity under the specified project deployment. Note that the inputs to the deployed model will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'holiday_yn' mapped to mapping 'FUTURE' in our system).    '''
         return self._call_api('getForecast', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'futureData': future_data, 'numPredictions': num_predictions, 'predictionStart': prediction_start})
 
-    def get_recommendations(self, deployment_token: str, deployment_id: str, query_data: dict, num_items: int = 50, page: int = 1, exclude_item_ids: list = [], score_field: str = '', scaling_factors: list = [], restrict_items: list = [], exclude_items: list = []):
+    def get_recommendations(self, deployment_token: str, deployment_id: str, query_data: dict, num_items: int = 50, page: int = 1, exclude_item_ids: list = [], score_field: str = '', scaling_factors: list = [], restrict_items: list = [], exclude_items: list = [], explore_fraction: float = 0.0):
         '''Returns a list of recommendations for a given user under the specified project deployment. Note that the inputs to this method, wherever applicable, will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'time' mapped to mapping 'TIMESTAMP' in our system).    '''
-        return self._call_api('getRecommendations', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'numItems': num_items, 'page': page, 'excludeItemIds': exclude_item_ids, 'scoreField': score_field, 'scalingFactors': scaling_factors, 'restrictItems': restrict_items, 'excludeItems': exclude_items})
+        return self._call_api('getRecommendations', 'POST', query_params={'deploymentToken': deployment_token, 'deploymentId': deployment_id}, body={'queryData': query_data, 'numItems': num_items, 'page': page, 'excludeItemIds': exclude_item_ids, 'scoreField': score_field, 'scalingFactors': scaling_factors, 'restrictItems': restrict_items, 'excludeItems': exclude_items, 'exploreFraction': explore_fraction})
 
     def get_personalized_ranking(self, deployment_token: str, deployment_id: str, query_data: dict, preserve_ranks: list = [], scaling_factors: list = []):
         '''Returns a list of items with personalized promotions on them for a given user under the specified project deployment. Note that the inputs to this method, wherever applicable, will be the column names in your dataset mapped to the column mappings in our system (e.g. column 'item_code' mapped to mapping 'ITEM_ID' in our system).    '''
@@ -486,7 +532,7 @@ class ApiClient():
         '''Retrieves the status of the specified batch prediction job.    '''
         return self._call_api('describeBatchPrediction', 'GET', query_params={'batchPredictionId': batch_prediction_id}, parse_type=BatchPrediction)
 
-    def add_user_item_interaction(self, streaming_token: str, dataset_id: str, timestamp: int, user_id: str, item_id: str, event_type: str, additional_attributes: dict):
+    def add_user_item_interaction(self, streaming_token: str, dataset_id: str, timestamp: int, user_id: str, item_id: list, event_type: str, additional_attributes: dict):
         '''Adds a user-item interaction record (data row) to a streaming dataset.    '''
         return self._call_api('addUserItemInteraction', 'POST', query_params={'streamingToken': streaming_token, 'datasetId': dataset_id}, body={'timestamp': timestamp, 'userId': user_id, 'itemId': item_id, 'eventType': event_type, 'additionalAttributes': additional_attributes})
 
