@@ -1,5 +1,6 @@
-from multiprocessing import Pool
 import io
+import time
+from multiprocessing import Pool
 
 
 class Upload():
@@ -45,9 +46,24 @@ class Upload():
     def describe(self):
         return self.client.describe_upload(self.upload_id)
 
+    def upload_part(self, part_number, part_data):
+        '''Uploads a file part.
+           If the upload fails, it will retry up to 3 times with a short backoff before raising an exception.
+        '''
+        retries = 0
+        while True:
+            try:
+                return self.part(part_number, part_data)
+            except Exception:
+                if retries > 2:
+                    raise
+                retries += 1
+                time.sleep(retries)
+
     def upload_file(self, file, threads=10, chunksize=1024 * 1024 * 10, wait_timeout=600):
         with Pool(processes=threads) as pool:
-            pool.starmap(self.part, self._yield_upload_part(file, chunksize))
+            pool.starmap(self.upload_part,
+                         self._yield_upload_part(file, chunksize))
         upload_object = self.mark_complete()
         self.wait_for_join(timeout=wait_timeout)
         if upload_object.batch_prediction_id:
